process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
const B2CAPI = require('b2c-api');
const totpGen = require('totp-generator');
const totp = totpGen('<<User TOTP Server key>>');

const url = '<<Trading API Endpoint>>';
const apiKey = '<<API Key>>';

const b2cAPI = new B2CAPI({
    url,
    apiKey,
    debug: true,
});

const userId = '<<User id>>';
const password = '<<Password>>';

(async () => {
    try {
        const logonResponse = await b2cAPI.login({ userId, password, totp });
        console.log(logonResponse);

        // message websocket
        b2cAPI.messageSocket.connect();

        b2cAPI.messageSocket.on('MSG:DATA', (response) => {
            console.log("message socket: data:", response);
        });

        b2cAPI.messageSocket.on('disconnect', (err) => {
            console.log("message socket: disconnect", err);
        });

        b2cAPI.messageSocket.on('connect_error', (err) => {
            console.log("message socket: connect_error", err);
        });

        b2cAPI.messageSocket.on('connect_failed', (err) => {
            console.log("message socket: connect_failed", err);
        });

        // broadcast websocket
        b2cAPI.broadcastSocket.connect();

        b2cAPI.broadcastSocket.on('open', (data) => {
            console.log('broadcast socket: open', data);
        });

        b2cAPI.broadcastSocket.on('close', (data) => {
            console.log('broadcast socket: close', data);
        });

        b2cAPI.broadcastSocket.on('loggedIn', (data) => {
            console.log('broadcast socket: loggedIn', data);
            b2cAPI.broadcastSocket.touchlineSub([{ 'mkt_seg_id': '1', 'scrip_token': '22' }, { 'mkt_seg_id': '1', 'scrip_token': '2885' }]);   //add market type
            b2cAPI.broadcastSocket.bestFiveSub({ 'mkt_seg_id': '1', 'scrip_token': '22' });
        });

        b2cAPI.broadcastSocket.on('error', (data) => {
            console.log('broadcast socket: error', data);
        });

        b2cAPI.broadcastSocket.on('touchline', (data) => {
            console.log('broadcast socket: touchline', data);
        });

        b2cAPI.broadcastSocket.on('bestfive', (data) => {
            console.log('broadcast socket: best five', data);
        });

        const placeNormalOrderParams = {
            'scrip_info': {
                'exchange': 'NSE_EQ',
                'scrip_token': 15124,
                'symbol': 'KOLTEPATIL',
                'series': 'EQ',
                'expiry_date': '',
                'strike_price': '',
                'option_type': ''
            },
            'transaction_type': 'SELL',
            'product_type': 'INTRADAY',
            'order_type': 'RL',
            'quantity': 10,
            'price': 300,
            'trigger_price': 0,
            'disclosed_quantity': 0,
            'validity': 'DAY',
            'validity_days': 0,
            'is_amo': true,
            'order_identifier': 'abc',
            'part_code': '',
            'algo_id': '',
            'strategy_id': '',
            'vender_code': ''
        };
        console.log('place normal order', await b2cAPI.placeOrder(placeNormalOrderParams));

        const modifyNormalOrderParams = {
            'exchange': 'NSE_EQ',
            'order_id': 'NXWCY0031176',
            'order_type': 'RL',
            'quantity': 5,
            'traded_quantity': 0,
            'price': 310,
            'trigger_price': 0,
            'disclosed_quantity': 0,
            'validity': 'DAY',
            'validity_days': 0
        };
        console.log('modify normal order', await b2cNode.modifyOrder(modifyNormalOrderParams));

        const cancelNormalOrderParams = {
            exchange: 'NSE_EQ',
            order_id: 'NXWCY00030I5'
        }
        console.log('cancel order', await b2cNode.cancelOrder(cancelNormalOrderParams));

        const coverOrderParams = {
            'scrip_info': {
                'exchange': 'NSE_EQ',
                'scrip_token': 15124,
                'symbol': 'KOLTEPATIL',
                'series': 'EQ',
                'expiry_date': '',
                'strike_price': '',
                'option_type': ''
            },
            'transaction_type': 'BUY',
            'main_leg': {
                'order_type': 'RL-MKT',
                'quantity': 3,
                'price': 0
            },
            'stoploss_leg': {
                'legs': [
                    {
                        'quantity': 0,
                        'price': 300,
                        'trigger_price': 300
                    }
                ]
            },
            'order_identifier': '',
            'part_code': '',
            'algo_id': '',
            'strategy_id': '',
            'vender_code': ''
        }
        console.log('place cover order', await b2cNode.placeCoverOrder(coverOrderParams));

        const modifyCoverOrderParams = {
            "exchange": "NSE_EQ",
            "order_id": "NXWCY0031776",
            "main_leg": {
                "order_type": "RL-MKT",
                "quantity": 5,
                "traded_quantity": 0,
                "price": 0
            },
            "stoploss_leg": {
                "legs": [
                    {
                        "quantity": 0,
                        "price": 305,
                        "trigger_price": 305
                    }
                ]
            }
        };
        console.log('modify cover order', await b2cNode.modifyCoverOrder(modifyCoverOrderParams));

        const cancelCoverOrderParams = {
            exchange: 'NSE_EQ',
            order_id: 'NXWCY0031776'
        }
        console.log('cancel cover order', await b2cNode.cancelCoverOrder(cancelCoverOrderParams));

        const bracketOrder = {
            'scrip_info': {
                'exchange': 'NSE_EQ',
                'scrip_token': 15124,
                'symbol': 'KOLTEPATIL',
                'series': 'EQ',
                'expiry_date': '',
                'strike_price': '',
                'option_type': ''
            },
            'transaction_type': 'BUY',
            'main_leg': {
                'order_type': 'RL',
                'quantity': 3,
                'price': '300',
                'trigger_price': 0,
                'traded_quantity': 0
            },
            'stoploss_leg': {
                'legs': [
                    {
                        'quantity': 0,
                        'price': 290,
                        'trigger_price': 290
                    }
                ],
                'trail': {
                    'ltp_jump_price': 0.50,
                    'stoploss_jump_price': 0.50
                }
            },
            'profit_leg': {
                'legs': [
                    {
                        'quantity': 0,
                        'price': 310
                    }
                ]
            },
            'order_identifier': '',
            'part_code': '',
            'algo_id': '',
            'strategy_id': '',
            'vender_code': ''
        }
        console.log('bracket order', await b2cNode.placeBracketOrder(bracketOrder));

        console.log('order book', await b2cNode.getOrderBook());
        console.log('trade book', await b2cNode.getTradeBook());
        console.log('positions', await b2cNode.getPositions({ type: 'all' }));
        console.log('holdings', await b2cNode.getHoldings());
    } catch (error) {
        console.log(error);
    }
})();