
const constants = require('./constants');
const EventEmitter = require('events').EventEmitter;

const commonMethods = {

    evtEmitter: null,

    getCurrentTime: function () {
        return `${new Date().toLocaleTimeString()} :`;
    },

    stringIsNullOrEmpty: function (data) {
        if (data == null || data == '') {
            return true;
        } else {
            return false;
        };
    },

    //BT-22235/BT-23226:PrajyotD:07-08-2020:RAF - Login Optimization <START>
    getDateTime: function () {
        let CurrentDate = new Date();
        let strTime = '';
        let hr, min, sec;
        hr = CurrentDate.getHours();
        min = CurrentDate.getMinutes();
        sec = CurrentDate.getSeconds();
        if (hr < 10)
            hr = '0' + hr;
        if (min < 10)
            min = '0' + min;
        if (sec < 10)
            sec = '0' + sec;
        strTime = hr + ':' + min + ':' + sec;
        return strTime;
    },

    convertToDecimal: function (sType, value, decimalLoc) {
        let returnVal = 0;
        let nfixed = decimalLoc == 1000 ? 4 : 2;
        try {
            if (value > 0) {
                if (sType == 'Price') returnVal = (parseFloat(value) / parseInt(decimalLoc)).toFixed(nfixed);
                else if (sType == 'Percentage') returnVal = parseFloat(value) / parseInt(decimalLoc);
            } else returnVal = value;
        } catch (error) {
            returnVal = '0';
        };
        return returnVal;
    },

    getTimeInSeconds: function () {
        const secondsSinceEpoch = Math.round(Date.now() / 1000)
        let timeInSeconds = secondsSinceEpoch;
        return timeInSeconds;
    },

    writeConsoleLog: function (data) {
        console.log(`${commonMethods.getCurrentTime()} ${data}`);
    },

    findValue: function (arrInput, strCheckFor) {
        let sVal = '';
        try {
            arrInput = '|' + arrInput;
            let myRegExp = new RegExp('[^0-9]' + strCheckFor.trim() + '[ ]*=[ ]*([^|]*)', 'g');
            let aRegResult = myRegExp.exec(arrInput);
            if (aRegResult !== null && aRegResult[1] != 'undefined' && aRegResult[1] != '') {
                sVal = aRegResult[1];
            };
        } catch (e) {
            throw e;
        };
        return sVal;
    },

    removeFieldDelimiter: function (_strSource) {
        if (_strSource.endsWith(constants.C_S_FIELD_DELIMITER))
            _strSource = _strSource.substring(0, _strSource.length - 1);
        return _strSource;
    },

    pubsub: function () {
        if (commonMethods.evtEmitter == null) {
            commonMethods.evtEmitter = new EventEmitter();
        };
        return commonMethods.evtEmitter;
    },

    // Returns an array of an comnbined result of KeyValuePair where key holds an msg tag and Value hold an data value related to the tag based on Field Delimiter
    lookUp: function (_strResponse) {
        let _lkUp = {};
        try {
            let sFieldData = '';
            let aFieldData;
            if (_strResponse != undefined) {
                let _respData = _strResponse.split(constants.C_S_FIELD_DELIMITER);
                for (let i = 0; i < _respData.length; i++) {
                    sFieldData = _respData[i];
                    if (sFieldData != 'undefined') {
                        aFieldData = sFieldData.split(constants.C_S_NAMEVALUE_DELIMITER);
                        _lkUp[aFieldData[0]] = aFieldData[1];
                    };
                };
            };
        } catch (e) {
        };
        return _lkUp;
    },

    /// To get the Price format as per the Decimal Locator, MktSegId, InstName using DecimalLocator, MktSegId
    getPriceFormatter: function (strDecimalLocator, intMktSegId) {
        try {
            if (intMktSegId == constants.C_V_MAPPED_MSX_DERIVATIVES || intMktSegId == constants.C_V_MAPPED_MSX_SPOT || intMktSegId == constants.C_V_MSX_DERIVATIVES || intMktSegId == constants.C_V_MSX_SPOT || intMktSegId == constants.C_V_NSX_DERIVATIVES || intMktSegId == constants.C_V_NSX_SPOT || intMktSegId == constants.C_V_MAPPED_NSX_DERIVATIVES || intMktSegId == constants.C_V_MAPPED_NSX_SPOT ||
                intMktSegId == constants.C_V_BSECDX_DERIVATIVES || intMktSegId == constants.C_V_BSECDX_SPOT || intMktSegId == constants.C_V_MAPPED_BSECDX_DERIVATIVES || intMktSegId == constants.C_V_MAPPED_BSECDX_SPOT || intMktSegId == constants.C_V_MAPPED_MCX_DERIVATIVES) {
                if (intMktSegId == constants.C_V_MAPPED_MCX_DERIVATIVES && strDecimalLocator.trim() == '0') {
                    strDecimalLocator = '100';
                };
                if (strDecimalLocator.trim() == '0') strDecimalLocator = '10000';
                if (strDecimalLocator === ('100')) return 2;
                else if (strDecimalLocator === ('10000') || strDecimalLocator === (constants.C_V_NSECDS_DECLOC)) return 4;
                else return 2;
            } else if (intMktSegId == constants.C_V_MAPPED_BFX_DERIVATIVES || intMktSegId == constants.C_V_MAPPED_BFX_SPOT) {
                if (strDecimalLocator === ('1000')) return 3;
                else if (strDecimalLocator === ('10000')) return 4;
                else return 2;
            } else if (intMktSegId == constants.C_V_MAPPED_MSX_CASH || intMktSegId == constants.C_V_MAPPED_MSX_FAO) {
                return this.getPriceFormatterFromDecimalLocator(parseInt(strDecimalLocator, 10));
            } else if (strDecimalLocator !== null && strDecimalLocator !== '' && strDecimalLocator !== '0') {
                return this.getNoOfZeros(strDecimalLocator);
            } else {
                return 2;
            };
        } catch (error) {
            // console.log(error);
        };
    },

    /**
     * Method to get mapped market segment id (manager once) from normal Market Segment Id (OC once)
     * e.g. for BSE if method input is 3 then method output will be 8
    */
    getMappedMarketSegmentId: function (_iMarketSegmentId) {
        let _iMappedMarketSegmentId = -1;
        try {
            switch (parseInt(_iMarketSegmentId)) {
                case constants.C_V_NSE_CASH:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NSE_CASH;
                    break;
                case constants.C_V_NSE_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NSE_DERIVATIVES;
                    break;
                case constants.C_V_BSE_CASH:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_BSE_CASH;
                    break;
                case constants.C_V_BSE_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_BSE_DERIVATIVES;
                    break;
                case constants.C_V_MCX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_MCX_DERIVATIVES;
                    break;
                case constants.C_V_MCX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_MCX_SPOT;
                    break;
                case constants.C_V_NCDEX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NCDEX_DERIVATIVES;
                    break;
                case constants.C_V_NCDEX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NCDEX_SPOT;
                    break;
                case constants.C_V_NSEL_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NSEL_DERIVATIVES;
                    break;
                case constants.C_V_NSEL_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NSEL_SPOT;
                    break;
                case constants.C_V_MSX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_MSX_DERIVATIVES;
                    break;
                case constants.C_V_MSX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_MSX_SPOT;
                    break;
                case constants.C_V_NSX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NSX_DERIVATIVES;
                    break;
                case constants.C_V_NSX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NSX_SPOT;
                    break;
                case constants.C_V_BSECDX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_BSECDX_DERIVATIVES;
                    break;
                case constants.C_V_BSECDX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_BSECDX_SPOT;
                    break;
                case constants.C_V_MSX_CASH:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_MSX_CASH;
                    break;
                case constants.C_V_MSX_FAO:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_MSX_FAO;
                    break;
                case constants.C_V_NMCE_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_NMCE_DERIVATIVES;
                    break;
                case constants.C_V_DSE_CASH:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_DSE_CASH;
                    break;
                case constants.C_V_UCX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_UCX_DERIVATIVES;
                    break;
                case constants.C_V_UCX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_UCX_SPOT;
                    break;
                case constants.C_V_DGCX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_DGCX_DERIVATIVES;
                    break;
                case constants.C_V_DGCX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_DGCX_SPOT;
                    break;
                case constants.C_V_BFX_DERIVATIVES:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_BFX_DERIVATIVES;
                    break;
                case constants.C_V_BFX_SPOT:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_BFX_SPOT;
                    break;
                case constants.C_V_OFS_IPO_BONDS:
                    _iMappedMarketSegmentId = constants.C_V_MAPPED_OFS_IPO_BONDS;
                    break;
                default:
                    _iMappedMarketSegmentId = -1;
                    break;
            }
        } catch (error) {
            // console.log(error);
        };
        return _iMappedMarketSegmentId;
    },

    /// To get the No. Of Zeros in the decimal locator
    getNoOfZeros: function (num) {
        num = parseFloat(num);
        let count = 0;
        let last = 0;
        while (last == 0) {
            last = parseFloat((num % 10).toString());
            num = num / 10;
            count++;
        };
        return count - 1;
    },

    parseTER: function (intMktSegId, strTER) {
        if (intMktSegId != constants.C_V_NSE_DERIVATIVES && intMktSegId != constants.C_V_NSX_DERIVATIVES && intMktSegId != constants.C_V_MAPPED_NSX_DERIVATIVES) {
            strTER = '';
        };
        return strTER;
    },

    getPriceFormatterFromDecimalLocator: function (iDecimalLc) {
        let strFormat = '2';
        try {
            strFormat = (iDecimalLc.toString().length - 1).toString();
        } catch (e) {
        };
        return parseInt(strFormat);
    },

    getPreciseValue: function (strOrgValue, MktSegId, DecimalLocator) {
        let strPrecValue = strOrgValue;
        let strPrecison = '2'; //Default Value as 2 precisions
        if (strOrgValue != '' && parseFloat(strOrgValue) > 0) {
            if (MktSegId == constants.C_V_MSX_DERIVATIVES || MktSegId == constants.C_V_MSX_SPOT || MktSegId == constants.C_V_NSX_SPOT || MktSegId == constants.C_V_NSX_DERIVATIVES ||
                MktSegId == constants.C_V_BSECDX_SPOT || MktSegId == constants.C_V_BSECDX_DERIVATIVES) {
                if (MktSegId == constants.C_V_NSX_SPOT || MktSegId == constants.C_V_BSECDX_SPOT) {
                    if (DecimalLocator == 100) {
                        strPrecison = '2';
                    } else {
                        strPrecison = '4';
                    };
                } else {
                    strPrecison = '4';
                };
            } else if (MktSegId == constants.C_V_MCX_SPOT || MktSegId == constants.C_V_NSEL_SPOT || MktSegId == constants.C_V_MCX_DERIVATIVES) {
                if (DecimalLocator == 10000)
                    strPrecison = '4';
            } else if (MktSegId == constants.C_V_MAPPED_BFX_DERIVATIVES || MktSegId == constants.C_V_MAPPED_BFX_SPOT
                || MktSegId == constants.C_V_BFX_DERIVATIVES || MktSegId == constants.C_V_BFX_SPOT) {
                if (DecimalLocator == 1000)
                    strPrecison = '3';
                if (DecimalLocator == 10000)
                    strPrecison = '4';
            } else if (MktSegId == constants.C_V_MSX_CASH || MktSegId == constants.C_V_MSX_FAO ||
                MktSegId == constants.C_V_MAPPED_MSX_CASH || MktSegId == constants.C_V_MAPPED_MSX_FAO) {
                let strFormat = '2';
                strFormat = (DecimalLocator.toString().length - 1).toString();
                strPrecison = strFormat;
            };
            strPrecValue = parseFloat(strOrgValue).toFixed(parseInt(strPrecison));
        };
        return strPrecValue;
    },


    getDateTimePart: function (_strDT) {
        if (this.Trim(_strDT) != '') {
            if (_strDT.indexOf(' ') == -1) {
                return _strDT;
            } else {
                _strDT = _strDT.split(' ');
                if (_strDT.length == 2) {
                    if (parseFloat(_strDT[1]) == 0) {
                        return '';
                    } else {
                        let str1 = this.Trim(_strDT[1]).substr(0, 2);
                        let str2 = this.Trim(_strDT[1]).substr(2, 2);
                        let str3 = this.Trim(_strDT[1]).substr(4, 2);
                        return str1 + ':' + str2 + ':' + str3 + ' ' + _strDT[0];
                    };
                } else {
                    return '';
                };
            };
        };
    },

    Trim: function (objTxtValue) {
        return objTxtValue.toString().split(' ').join('');
    },
};

module.exports = commonMethods;
