const constants = require('./constants');
const commonMethods = require('./common-methods');
const StringBuilder = require('string-builder');

class APIParser {
    constructor() {
        const self = this;
        this.maskMsgLength = '#PACKET_LENGTH#';

        self.createRequestMessageHeader = function (_messageCode) {

            try {
                let _sbMessageHeader = new StringBuilder();

                let _currentTime = commonMethods.getDateTime();
                _sbMessageHeader.append(constants.C_S_TAG_MSGVERSION + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_TAG_COMMPROTOCOL + constants.C_S_FIELD_DELIMITER);
                _sbMessageHeader.append(constants.C_S_TAG_MSGCODE + constants.C_S_NAMEVALUE_DELIMITER + _messageCode + constants.C_S_FIELD_DELIMITER);
                _sbMessageHeader.append(constants.C_S_TAG_MSGLENGTH + constants.C_S_NAMEVALUE_DELIMITER + this.maskMsgLength + constants.C_S_FIELD_DELIMITER);
                _sbMessageHeader.append(constants.C_S_TAG_MSGTIME + constants.C_S_NAMEVALUE_DELIMITER + _currentTime + constants.C_S_FIELD_DELIMITER);

                return _sbMessageHeader.toString();

            } catch (error) {
                //clsGlobal.logManager.writeLog(clsGlobal.ComId, constants.C_S_LOGTYPE_APPERROR, undefined, clsGlobal.logManager.createLogPayload('clsParser', 'createRequestMessageHeader', error.Message, undefined, error.stack, undefined, undefined));
            }
        };

        self.calculateMessageLength = function (_request) {
            try {
                let _intPktLength = _request.length;
                let _strPktLength = (_intPktLength + _intPktLength.toString().length - this.maskMsgLength.length).toString();
                return _request.replace(this.maskMsgLength, _strPktLength);
            }
            catch (error) {
                //clsGlobal.logManager.writeLog(clsGlobal.ComId, constants.C_S_LOGTYPE_APPERROR, undefined, clsGlobal.logManager.createLogPayload('clsParser', 'calculateMessageLength', error.Message, undefined, error.stack, undefined, undefined));
            }
        };

        self.createLoginRequest = function (reqLogin) {
            try {
                let _sbFinalRequest = new StringBuilder();
                _sbFinalRequest.append(this.createRequestMessageHeader(constants.C_V_MSGCODES_LOGINREQUEST));
                _sbFinalRequest.append(constants.C_S_TAG_USERID + constants.C_S_NAMEVALUE_DELIMITER + reqLogin.userId + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_PWD + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_FIELD_DELIMITER);
                // _sbFinalRequest.append(constants.C_S_TAG_PWD + constants.C_S_NAMEVALUE_DELIMITER + reqLogin.token + constants.C_S_FIELD_DELIMITER); // TODO
                _sbFinalRequest.append(constants.C_S_CONNECTIONTYPE + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_V_TAG_IWINLOGONTAG + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_NETNETLOGONTAG + constants.C_S_FIELD_DELIMITER);
                // _sbFinalRequest.append(constants.C_V_TAG_PWD_TYPE + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_ENCYP_TYPE_TOKEN + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_V_TAG_PWD_TYPE + constants.C_S_NAMEVALUE_DELIMITER + '1' + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_GROUPID + constants.C_S_NAMEVALUE_DELIMITER + 'HO' + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_CONNECTION_TYPE + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_CONNECTIONTYPE + constants.C_S_FIELD_DELIMITER);
                // _sbFinalRequest.append(constants.C_V_TAG_TENANTID + constants.C_S_NAMEVALUE_DELIMITER + reqLogin.tenantId + constants.C_S_FIELD_DELIMITER);
                // _sbFinalRequest.append('395=127.0.0.1');
                let _strFinalRequest = this.calculateMessageLength(_sbFinalRequest.toString());
                return _strFinalRequest;
            } catch (error) {
                // commonMethods.writeConsoleLog('Error in createLoginRequest: ' + error.message);
            }
        };

        self.createTouchLineRequest = function (_operationType, _lstTLRequest, FullLtpReq) {
            try {
                let _sbFinalRequest = new StringBuilder();
                _sbFinalRequest.append(this.createRequestMessageHeader(constants.C_S_MSGCODE_MULTIPLE_TOUCHLINE_REQUEST));
                //_sbFinalRequest.append(constants.C_S_TAG_SESSIONID + constants.C_S_NAMEVALUE_DELIMITER + clsGlobal.User.OCToken + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_SESSIONID + constants.C_S_NAMEVALUE_DELIMITER + constants.C_S_FIELD_DELIMITER); // TODO

                //_sbFinalRequest.append(constants.C_V_TAG_TENANTID + constants.C_S_NAMEVALUE_DELIMITER + clsGlobal.ComId + constants.C_S_FIELD_DELIMITER);
                for (let i = 0; i < _lstTLRequest.length; i++) {
                    _sbFinalRequest.append(constants.C_S_TAG_MKTSEGID + constants.C_S_NAMEVALUE_DELIMITER + _lstTLRequest[i].mkt_seg_id);
                    _sbFinalRequest.append(constants.C_S_RECORD_DELIMITER);
                    _sbFinalRequest.append(constants.C_S_TAG_SCRIPTOKEN + constants.C_S_NAMEVALUE_DELIMITER + _lstTLRequest[i].scrip_token);
                    _sbFinalRequest.append(constants.C_S_FIELD_DELIMITER);
                };
                if (!FullLtpReq) {
                    _sbFinalRequest.append(constants.C_S_TAG_CUMULATIVEOI + constants.C_S_NAMEVALUE_DELIMITER + '1' + constants.C_S_FIELD_DELIMITER);
                };
                _sbFinalRequest.append(constants.C_S_TAG_OPERATIONTYPE + constants.C_S_NAMEVALUE_DELIMITER + _operationType);
                //_sbFinalRequest.append(constants.C_S_FIELD_DELIMITER);
                //_sbFinalRequest.append(constants.C_V_TAG_SI_TEMPLATEID + constants.C_S_NAMEVALUE_DELIMITER + clsGlobal.User.SITemplateId);
                let _strFinalRequest = this.calculateMessageLength(_sbFinalRequest.toString());
                return _strFinalRequest;
            } catch (error) {
            };
        };

        self.createBestFiveRequest = function (_objBestFiveReq) {
            try {
                let _sbFinalRequest = new StringBuilder();
                _sbFinalRequest.append(this.createRequestMessageHeader(constants.C_S_MSGCODE_BESTFIVE_REQUEST));
                //_sbFinalRequest.append(constants.C_S_TAG_SESSIONID + constants.C_S_NAMEVALUE_DELIMITER + clsGlobal.User.OCToken + constants.C_S_FIELD_DELIMITER);
                //_sbFinalRequest.append(constants.C_V_TAG_TENANTID + constants.C_S_NAMEVALUE_DELIMITER + clsGlobal.ComId + constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_SCRIPTOKEN + constants.C_S_NAMEVALUE_DELIMITER + _objBestFiveReq.scrip_token);
                _sbFinalRequest.append(constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_MKTSEGID + constants.C_S_NAMEVALUE_DELIMITER + _objBestFiveReq.mkt_seg_id);
                _sbFinalRequest.append(constants.C_S_FIELD_DELIMITER);
                _sbFinalRequest.append(constants.C_S_TAG_OPERATIONTYPE + constants.C_S_NAMEVALUE_DELIMITER + _objBestFiveReq.operation);
                //_sbFinalRequest.append(constants.C_S_FIELD_DELIMITER);
                //_sbFinalRequest.append(constants.C_V_TAG_SI_TEMPLATEID + constants.C_S_NAMEVALUE_DELIMITER + clsGlobal.User.SITemplateId);
                let _strFinalRequest = this.calculateMessageLength(_sbFinalRequest.toString());
                return _strFinalRequest;
            }
            catch (error) {
                // clsGlobal.logManager.writeLog(clsGlobal.ComId, constants.C_S_LOGTYPE_APPERROR, undefined, clsGlobal.logManager.createLogPayload('clsParser', 'createBestFiveRequest', error.Message, undefined, error.stack, undefined, undefined));
            }
        };

        self.processLogonResponse = function (_responsePacket, _channelId) {

            let objResp = {};
            let _strLoginResponse = '';
            let _sMsgCode = '';
            let _sAuthCode = '';
            try {
                //objResp = new clsOnlineResponse();
                _sMsgCode = commonMethods.findValue(_responsePacket, constants.C_S_TAG_MSGCODE);
                let _resArray = commonMethods.lookUp(_responsePacket);

                _sAuthCode = _resArray[constants.C_S_TAG_AUTHCODE];
                if (_sAuthCode !== '') {
                    //TODO:
                    //_strLoginResponse = ' ' + _channelId + ' ' + clsGlobal.dMsgMaster['NNSL98');
                    objResp.MsgCode = _sMsgCode;
                    objResp.MsgData = 'Login Success';
                    objResp.MsgTime = commonMethods.getCurrentTime();
                    objResp.MsgCategory = constants.C_S_MSGCAT_ACK;
                    //objResp.OrderSide = constants.C_S_MSGCAT_LOGIN;    //For color purpose
                }
                return objResp;
            } catch (error) {
                //clsGlobal.logManager.writeLog(clsGlobal.ComId, constants.C_S_LOGTYPE_APPERROR, undefined, clsGlobal.logManager.createLogPayload('clsParser', 'processLogonResponse', error.Message, undefined, error.stack, undefined, undefined));
            };
        };

        self.processMultiTouchLineResponse = function (_responsePacket) {

            let _objMultiTLResponse = {};
            try {
                let lstResponse = commonMethods.lookUp(_responsePacket);
                let skScrip = {};
                skScrip.mkt_seg_id = parseInt(lstResponse[(constants.C_S_TAG_MKTSEGID)]);
                let MapMktSegId = commonMethods.getMappedMarketSegmentId(skScrip.mkt_seg_id);
                skScrip.scrip_token = lstResponse[(constants.C_S_TAG_SCRIPTOKEN)];
                //_objMultiTLResponse = clsResponseStore.getTouchLineResponseObject(skScrip.toString(), true);
                //if response received for scrip which is removed from request store then return null
                if (_objMultiTLResponse == null) {
                    return _objMultiTLResponse;
                }

                _objMultiTLResponse.scrip = skScrip;
                _objMultiTLResponse.last_updated_time = lstResponse[(constants.C_S_TAG_LUT)];

                let intDecimalLocator = 100;

                if (lstResponse[(constants.C_S_TAG_DECIMALLOCATOR)] == '' || lstResponse[(constants.C_S_TAG_DECIMALLOCATOR)] == '0')
                    intDecimalLocator = 100;

                else
                    intDecimalLocator = parseInt(lstResponse[(constants.C_S_TAG_DECIMALLOCATOR)]);

                let _strPriceFormat = commonMethods.getPriceFormatter(intDecimalLocator.toString(), MapMktSegId);

                //_objMultiTLResponse.price_format = _strPriceFormat;
                _objMultiTLResponse.decimal_locator = intDecimalLocator;
                //these tags will always be present in touchline Response whether CCast On or OFF so directly accessing value from list
                _objMultiTLResponse.last_traded_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_LTP)], intDecimalLocator, _strPriceFormat);
                if (_objMultiTLResponse.last_traded_price == '')
                    _objMultiTLResponse.last_traded_price = '0.00';

                _objMultiTLResponse.net_change_in_perc = (parseFloat(lstResponse[(constants.C_S_TAG_NETCHANGEFROMPREVCLOSE)])).toFixed(2);

                _objMultiTLResponse.last_traded_quantity = (this.qtyFormatter(lstResponse[(constants.C_S_TAG_LTQ)], 0)).toString();
                _objMultiTLResponse.last_traded_time = lstResponse[(constants.C_S_TAG_LTT)];

                _objMultiTLResponse.buy_quantity = (this.qtyFormatter(lstResponse[(constants.C_S_TAG_BUYQTY)], 0)).toString();
                _objMultiTLResponse.buy_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_BUYPRICE)], intDecimalLocator, _strPriceFormat);

                _objMultiTLResponse.sell_quantity = (this.qtyFormatter(lstResponse[(constants.C_S_TAG_SELLQTY)], 0)).toString();
                _objMultiTLResponse.sell_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_SELLPRICE)], intDecimalLocator, _strPriceFormat);

                _objMultiTLResponse.volume = (this.qtyFormatter(lstResponse[(constants.C_S_TAG_VOLUME)], 0)).toString();
                _objMultiTLResponse.close_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_CLOSEPRICE)], intDecimalLocator, _strPriceFormat);
                _objMultiTLResponse.average_trade_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_ATP)], intDecimalLocator, _strPriceFormat);

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_NETCHANGEINRS))
                    _objMultiTLResponse.net_change_in_rs = lstResponse[(constants.C_S_TAG_NETCHANGEINRS)];

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_OPENPRICE))
                    _objMultiTLResponse.open_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_OPENPRICE)], intDecimalLocator, _strPriceFormat);

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_HIGHPRICE))
                    _objMultiTLResponse.high_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_HIGHPRICE)], intDecimalLocator, _strPriceFormat);

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_LOWPRICE))
                    _objMultiTLResponse.low_price = this.priceFormatter(lstResponse[(constants.C_S_TAG_LOWPRICE)], intDecimalLocator, _strPriceFormat);

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_OPENINTEREST))
                    _objMultiTLResponse.open_interest = lstResponse[(constants.C_S_TAG_OPENINTEREST)];

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_TOTBUYQTY)) {
                    // changed to double as for index token in TotBuyQty MktCapitalization will be received
                    try {
                        _objMultiTLResponse.total_buy_quantity = (this.qtyFormatter(lstResponse[(constants.C_S_TAG_TOTBUYQTY)], 1)).toString();
                    }
                    catch (e) {
                    };
                }
                if (lstResponse.hasOwnProperty(constants.C_S_TAG_TOTSELLQTY))
                    _objMultiTLResponse.total_sell_quantity = (this.qtyFormatter(lstResponse[(constants.C_S_TAG_TOTSELLQTY)], 0)).toString();

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_LIFETIMEHIGH))
                    _objMultiTLResponse.life_time_high = (parseFloat(lstResponse[(constants.C_S_TAG_LIFETIMEHIGH)]) / intDecimalLocator).toFixed(_strPriceFormat);

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_LIFETIMELOW))
                    _objMultiTLResponse.life_time_low = (parseFloat(lstResponse[(constants.C_S_TAG_LIFETIMELOW)]) / intDecimalLocator).toFixed(_strPriceFormat);

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_DPR))
                    _objMultiTLResponse.daily_price_range = lstResponse[(constants.C_S_TAG_DPR)];

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_PERC_OPENINTEREST))
                    _objMultiTLResponse.perc_open_int = lstResponse[(constants.C_S_TAG_PERC_OPENINTEREST)];

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_HIGH_OPENINTEREST))
                    _objMultiTLResponse.high_open_int = lstResponse[(constants.C_S_TAG_HIGH_OPENINTEREST)];

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_LOW_OPENINTEREST))
                    _objMultiTLResponse.low_open_int = lstResponse[(constants.C_S_TAG_LOW_OPENINTEREST)];

                if (lstResponse.hasOwnProperty(constants.C_S_TAG_TRADE_EXECUTION_RANGE))
                    _objMultiTLResponse.trade_execution_range = commonMethods.parseTER(skScrip.mkt_seg_id, lstResponse[(constants.C_S_TAG_TRADE_EXECUTION_RANGE)]);

                lstResponse = null;
                skScrip = null;
                _responsePacket = null;

                return _objMultiTLResponse;
            }
            catch (error) {
                // console.log(error);
                //clsGlobal.logManager.writeLog(clsGlobal.ComId, constants.C_S_LOGTYPE_APPERROR, undefined, clsGlobal.logManager.createLogPayload('clsParser', 'processMultiTouchLineResponse', error.Message, undefined, error.stack, undefined, undefined));
            };
        };

        self.processBestFiveResponse = function (_responsePacket) {
            let _objBestFiveResponse = null;
            try {

                let strRespData = [];
                let iLResp = commonMethods.lookUp(_responsePacket);
                let strSegmentId = iLResp[constants.C_S_TAG_MKTSEGID];
                let strToken = iLResp[constants.C_S_TAG_SCRIPTOKEN];

                let objScrip = {};
                objScrip.mkt_seg_id = strSegmentId != '' ? parseInt(strSegmentId) : -1;
                let MapMktSegId = commonMethods.getMappedMarketSegmentId(objScrip.mkt_seg_id);
                objScrip.scrip_token = strToken;

                //gets the Best5 object (new or existing) from response store based on Scrip key
                _objBestFiveResponse = {};

                _objBestFiveResponse.scrip = objScrip;
                let intDecimalLocator = 100;
                let decDecLocator = 100;
                if (iLResp[constants.C_S_TAG_DECIMALLOCATOR] == '' || iLResp[constants.C_S_TAG_DECIMALLOCATOR] == '0' || iLResp[constants.C_S_TAG_DECIMALLOCATOR] == '100.00')
                    intDecimalLocator = 100;
                else {
                    decDecLocator = parseFloat(iLResp[constants.C_S_TAG_DECIMALLOCATOR]); //Used decimal variable to handle the case of MCXSX where the Decimal locator was '10000.00'
                    intDecimalLocator = decDecLocator;
                }
                let _strPriceFormat = commonMethods.getPriceFormatter(intDecimalLocator.toString(), MapMktSegId); //intDecimalLocator.toString().length - 1;

                let strVolume = iLResp[constants.C_S_TAG_VOLUME];
                let strOpenPrice = iLResp[constants.C_S_TAG_OPENPRICE];
                let strClosePrice = iLResp[constants.C_S_TAG_CLOSEPRICE];

                let strPerChg = iLResp[constants.C_S_TAG_NETCHANGEFROMPREVCLOSE];
                let strTBQ = iLResp[constants.C_S_TAG_TOTBUYQTY];
                let strDPR = iLResp[constants.C_S_TAG_DPR];
                let strLTT = iLResp[constants.C_S_TAG_LTT];
                let strLUT = iLResp[constants.C_S_TAG_LUT];

                let strHighPrice = iLResp[constants.C_S_TAG_HIGHPRICE];
                let strLowPrice = iLResp[constants.C_S_TAG_LOWPRICE];
                let strLTQ = iLResp[constants.C_S_TAG_LTQ];
                let strLTP = iLResp[constants.C_S_TAG_LTP];

                let strYearHighPrice = iLResp[constants.C_S_TAG_LIFETIMEHIGH];
                let strYearLowPrice = iLResp[constants.C_S_TAG_LIFETIMELOW];

                let strTSQ = iLResp[constants.C_S_TAG_TOTSELLQTY];
                let strATP = iLResp[constants.C_S_TAG_ATP];

                _responsePacket = _responsePacket.replace(/\|12/g, '&12');
                strRespData = _responsePacket.split('|');

                let B5 = '';
                let S5 = '';
                for (let i = 0; i < strRespData.length; i++) {
                    if (strRespData[i].length > 0) {
                        if (strRespData[i].indexOf('&') == -1) {
                            //dont know what have been done here find out later
                            //let sdata = strRespData[i].split('=');
                            continue;
                        }
                        else {
                            if (strRespData[i].startsWith('11=1'))
                                B5 = strRespData[i];
                            if (strRespData[i].startsWith('11=2'))
                                S5 = strRespData[i];
                        }
                    }
                }

                /* First Split entire Buy and Sell data with respect to &
                * We will have length of array equal to 6  */
                let strBuyData = B5.split('&');
                let strSellData = S5.split('&');

                /* Iterate through above Buy and Sell data arrays
                * starting from 1st index, since 0th index has BUY and Sell indication only
                * NOTE: Here Length of strBuyData and strSellData will be same hence use any one array length */
                let strB1 = '';
                let strS1 = '';
                _objBestFiveResponse.best_five_data = [{}, {}, {}, {}, {}];
                for (let i = 1; i < strBuyData.length; i++) {
                    /* Split individual data w.r.t. $ */
                    if (strBuyData[i] !== undefined)
                        strB1 = strBuyData[i].split('$');
                    if (strSellData[i] !== undefined)
                        strS1 = strSellData[i].split('$');

                    let dicBuyData = {};
                    let dicSellData = {};
                    // Further Split Buy data record w.r.t '=' to segregate No of Buyers, Price, Quantity
                    for (let j = 0; j < strB1.length; j++) {
                        let sdataB = strB1[j].split('=');
                        dicBuyData[sdataB[0]] = sdataB[1];
                    }

                    // Further Split Sell data record w.r.t '=' to segregate No of Sellers, Price, Quantity
                    for (let k = 0; k < strS1.length; k++) {
                        let sdataS = strS1[k].split('=');
                        dicSellData[sdataS[0]] = sdataS[1];
                    }

                    // Fill properties of B5DetailData (Buy and Sell data together as single row)
                    //Normal case
                    _objBestFiveResponse.best_five_data[i - 1].bid_price = dicBuyData[constants.C_S_TAG_ORDPRICE] != '' && parseFloat(dicBuyData[constants.C_S_TAG_ORDPRICE]) > 0 ? commonMethods.getPreciseValue((parseFloat(dicBuyData[constants.C_S_TAG_ORDPRICE]) / intDecimalLocator).toFixed(_strPriceFormat), parseInt(strSegmentId), intDecimalLocator) : '-';
                    _objBestFiveResponse.best_five_data[i - 1].bid_quantity = dicBuyData[constants.C_S_TAG_ORIGQTY] != '' ? parseInt(dicBuyData[constants.C_S_TAG_ORIGQTY]) > 0 ? dicBuyData[constants.C_S_TAG_ORIGQTY] : '-' : '-';
                    _objBestFiveResponse.best_five_data[i - 1].buyers = dicBuyData[constants.C_S_TAG_BEST5NOOFORDERS] != '' ? parseInt(dicBuyData[constants.C_S_TAG_BEST5NOOFORDERS]) > 0 ? dicBuyData[constants.C_S_TAG_BEST5NOOFORDERS] : '-' : '-';
                    _objBestFiveResponse.best_five_data[i - 1].ask_price = dicSellData[constants.C_S_TAG_ORDPRICE] != '' && parseFloat(dicSellData[constants.C_S_TAG_ORDPRICE]) > 0 ? commonMethods.getPreciseValue((parseFloat(dicSellData[constants.C_S_TAG_ORDPRICE]) / intDecimalLocator).toFixed(_strPriceFormat), parseInt(strSegmentId), intDecimalLocator) : '-';
                    _objBestFiveResponse.best_five_data[i - 1].ask_quantity = dicSellData[constants.C_S_TAG_ORIGQTY] != '' ? parseInt(dicSellData[constants.C_S_TAG_ORIGQTY]) > 0 ? dicSellData[constants.C_S_TAG_ORIGQTY] : '-' : '-';
                    _objBestFiveResponse.best_five_data[i - 1].sellers = dicSellData[constants.C_S_TAG_BEST5NOOFORDERS] != '' ? parseInt(dicSellData[constants.C_S_TAG_BEST5NOOFORDERS]) > 0 ? dicSellData[constants.C_S_TAG_BEST5NOOFORDERS] : '-' : '-';

                    //Clear the dictionary to facilitate its filling on next iteration
                    dicBuyData = null;
                    dicSellData = null;
                }
                _objBestFiveResponse.decimal_locator = intDecimalLocator;
                _objBestFiveResponse.last_updated_time = strLUT;
                _objBestFiveResponse.volume = strVolume != '' ? parseInt(strVolume) > 0 ? strVolume : '' : '';

                // Normal Case
                if (!_objBestFiveResponse.IsSpread) {
                    _objBestFiveResponse.open_price = strOpenPrice != '' ? parseFloat(strOpenPrice) > 0 ? (parseFloat(strOpenPrice) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';
                    _objBestFiveResponse.close_price = strClosePrice != '' ? parseFloat(strClosePrice) > 0 ? (parseFloat(strClosePrice) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';
                    _objBestFiveResponse.high_price = strHighPrice != '' ? parseFloat(strHighPrice) > 0 ? (parseFloat(strHighPrice) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';
                    _objBestFiveResponse.low_price = strLowPrice != '' ? parseFloat(strLowPrice) > 0 ? (parseFloat(strLowPrice) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';
                    _objBestFiveResponse.last_traded_price = strLTP != '' ? parseFloat(strLTP) > 0 ? (parseFloat(strLTP) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';
                }
                else {
                    _objBestFiveResponse.open_price = strOpenPrice != '' ? strOpenPrice != '0' ? (parseFloat(strOpenPrice) / intDecimalLocator).toString() : '' : '';
                    _objBestFiveResponse.close_price = strClosePrice != '' ? strClosePrice != '0' ? (parseFloat(strClosePrice) / intDecimalLocator).toString() : '' : '';
                    _objBestFiveResponse.high_price = strHighPrice != '' ? strHighPrice != '0' ? (parseFloat(strHighPrice) / intDecimalLocator).toString() : '' : '';
                    _objBestFiveResponse.low_price = strLowPrice != '' ? strLowPrice != '0' ? (parseFloat(strLowPrice) / intDecimalLocator).toString() : '' : '';
                    _objBestFiveResponse.last_traded_price = strLTP != '' ? strLTP != '0' ? (parseFloat(strLTP) / intDecimalLocator).toString() : '' : '';
                }

                if (commonMethods.Trim(strPerChg) == '-100' || commonMethods.Trim(strPerChg) == '-100.00') //Condition handled for preopen
                    strPerChg = '';
                _objBestFiveResponse.net_change_in_perc = strPerChg;
                _objBestFiveResponse.total_buy_quantity = strTBQ != '' ? parseInt(strTBQ) > 0 ? strTBQ : '' : '';
                _objBestFiveResponse.daily_price_range = strDPR;
                _objBestFiveResponse.last_traded_time = commonMethods.getDateTimePart(strLTT);
                _objBestFiveResponse.last_updated_time = commonMethods.getDateTimePart(strLUT);

                _objBestFiveResponse.last_traded_quantity = strLTQ != '' ? parseInt(strLTQ) > 0 ? strLTQ : '' : '';
                _objBestFiveResponse.net_change_in_perc = _objBestFiveResponse.last_traded_price != '' ? strPerChg : ''; //If LTP is present then display Percentage change else blank
                _objBestFiveResponse.year_high_price = strYearHighPrice != '' ? parseFloat(strYearHighPrice) > 0 ? (parseFloat(strYearHighPrice) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';
                _objBestFiveResponse.year_low_price = strYearLowPrice != '' ? parseFloat(strYearLowPrice) > 0 ? (parseFloat(strYearLowPrice) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';

                _objBestFiveResponse.total_sell_quantity = strTSQ != '' ? parseInt(strTSQ) > 0 ? strTSQ : '' : '';
                _objBestFiveResponse.average_trade_price = strATP != '' ? parseFloat(strATP) > 0 ? (parseFloat(strATP) / intDecimalLocator).toFixed(_strPriceFormat) : '' : '';

                _objBestFiveResponse.trade_execution_range = commonMethods.parseTER(objScrip.mkt_seg_id, iLResp[constants.C_S_TAG_TRADE_EXECUTION_RANGE]);
                //_objBestFiveResponse.trade_execution_range = commonMethods.ParseTER(objScrip.ScripDet.mkt_seg_id, iLResp[constants.C_S_TAG_TRADE_EXECUTION_RANGE]);
                if (iLResp.hasOwnProperty(constants.C_S_TAG_PERC_OPENINTEREST))
                    _objBestFiveResponse.open_interesterest = iLResp[constants.C_V_TAG_OPENINTEREST.toString()];

                if (iLResp.hasOwnProperty(constants.C_S_TAG_PERC_OPENINTEREST))
                    _objBestFiveResponse.perc_open_int = iLResp[constants.C_S_TAG_PERC_OPENINTEREST];

                if (iLResp.hasOwnProperty(constants.C_S_TAG_HIGH_OPENINTEREST))
                    _objBestFiveResponse.high_open_int = iLResp[constants.C_S_TAG_HIGH_OPENINTEREST];

                if (iLResp.hasOwnProperty(constants.C_S_TAG_LOW_OPENINTEREST))
                    _objBestFiveResponse.low_open_int = iLResp[constants.C_S_TAG_LOW_OPENINTEREST];

                let NetChangeInRs = 0;
                if (_objBestFiveResponse.last_traded_price != '' && _objBestFiveResponse.close_price != '')
                    NetChangeInRs = parseFloat(_objBestFiveResponse.last_traded_price) - parseFloat(_objBestFiveResponse.close_price);

                _objBestFiveResponse.net_change_in_rs = NetChangeInRs != '' ? NetChangeInRs.toFixed(_strPriceFormat) : '';
                _objBestFiveResponse.price_format = _strPriceFormat;

                iLResp = null;
                objScrip = null;
                _responsePacket = null;
                strRespData = null;

                return _objBestFiveResponse;

            } catch (error) {
                // console.log(error);
                //clsGlobal.logManager.writeLog(clsGlobal.ComId, constants.C_S_LOGTYPE_APPERROR, undefined, clsGlobal.logManager.createLogPayload('clsParser', 'processBestFiveResponse', e.Message, undefined, e.stack, undefined, undefined));
            }
        };

        self.priceFormatter = function (_sPriceVal, _intDecLoc, _strPriceFormat) {
            let _decPrice = parseFloat(_sPriceVal);
            return ((_decPrice <= 0) ? (parseInt('0')).toFixed(_strPriceFormat) : ((_decPrice / _intDecLoc).toFixed(_strPriceFormat))); // Spread related check implementation is pending
        };

        self.qtyFormatter = function (_sQtyVal, _intValueIn) {

            if (_intValueIn == 0)
                return (parseInt(_sQtyVal) <= 0) ? 0 : parseInt(_sQtyVal);
            else if (_intValueIn == 1)
                return (parseFloat(_sQtyVal) <= 0) ? 0 : parseFloat(_sQtyVal);
        };

    }
}

module.exports = APIParser;