const moment = require('moment');

const mappingMsgSocResp = {
    UniqueCode: {
        property: 'order_id'
    },
    Exchange: {
        property: 'exchange',
        values: {
            1: 'NSE_EQ',
            2: 'NSE_FO',
            3: 'BSE_EQ',
            4: 'BSE_FO',
            5: 'MCX_FO',
            6: 'MCX_SPOT',
            7: 'NCDEX_FO',
            8: 'NCDEX_SPOT',
            9: 'NSEL_SPTCOM',
            // 10: '',
            11: 'MSE_CUR',
            12: 'MSE_SPOT',
            13: 'NSE_CUR',
            14: 'NSECUR_SPOT',
            15: 'MSE_EQ',
            16: 'MSE_FO',
            // 17: '',
            18: 'BSE_COMM',
            19: 'NSE_COMM',
            20: 'NSECOMM_SPOT',
            // 21: '',
            // 25: '',
            // 26: '',
            // 27: '',
            // 31: '',
            // 32: '',
            33: 'NSE_OTS',
            34: 'ICEX_FO',
            // 36: '',
            // 37: '',
            38: 'BSE_CUR',
            39: 'BSECUR_SPOT',
            40: 'BSECOMM_SPOT',
            // 100: '',
            521: 'EQ Combined',
            4102: 'FNO Combined',
            268438528: 'CDS Combined',

        }
    },
    ScripCode: {
        property: 'scrip_token',
    },
    OrderNumber: {
        property: 'exchange_order_no'
    },
    OrderStatus: {
        property: 'status',
        values: {
            '-1': 'NOT_INITIATED',
            '1': 'CLIENT_XMITTED',
            '2': 'GATEWAY_XMITTED',
            '3': 'OMS_XMITTED',
            '4': 'EXCHANGE_XMITTED',
            '5': 'PENDING',
            '6': 'CANCELLED',
            '7': 'EXECUTED',
            '8': 'ADMIN_PENDING',
            '9': 'GATEWAY_REJECT',
            '10': 'OMS_REJECT',
            '11': 'ORDER_ERROR',
            '12': 'FROZEN',
            '13': 'M.PENDING',
            '14': 'ADMIN_ACCEPT',
            '15': 'ADMIN_REJECT',
            '16': 'ADMIN_MODIFY',
            '17': 'ADMIN_CANCEL',
            '18': 'AMO_SUBMITTED',
            '19': 'AMO_CANCELLED',
            '20': 'COMPLETED',
            '21': 'STOPPED',
            '22': 'CONVERTED',
        }
    },
    Reason: {
        property: 'error_reason'
    },
    Buy_Sell: {
        property: 'transaction_type',
        values: {
            1: 'BUY',
            2: 'SELL'
        }
    },
    Product: {
        property: 'product_type',
        values: {
            D: 'DELIVERY',
            M: 'INTRADAY',
            MF: 'MTF',
            PT: 'BTST',
            MP: 'COVER',
            B: 'BRACKET',
            AD: 'DELIVERY',
            AM: 'INTRADAY',
        }
    },
    OrderType: {
        property: 'order_type',
        values: {
            1: 'RL',
            2: 'RL-MKT',
            3: 'SL',
            4: 'SL-MKT',
        }
    },
    OrderOriginalQty: {
        property: 'total_quantity'
    },
    PendingQty: {
        property: 'pending_quantity'
    },
    TradedQTY: {
        property: 'traded_quantity'
    },
    DQ: {
        property: 'disclosed_quantity'
    },
    OrderPrice: {
        property: 'order_price'
    },
    TriggerPrice: {
        property: 'trigger_price'
    },
    OrderValidity: {
        property: 'validity',
    },
    Days: {
        property: 'validity_days',
    },
    Symbol: {
        property: 'symbol'
    },
    Series: {
        property: 'series'
    },
    InstrumentName: {
        property: 'instrument'
    },
    ExpiryDate: {
        property: 'expiry_date'
    },
    StrikePrice: {
        property: 'strike_price'
    },
    Option_Type: {
        property: 'option_type'
    },
    OrderEntryTime: {
        property: 'order_timestamp'
    },
    LastModifiedTime: {
        property: 'exchange_timestamp'
    },
    InitiatedBy: {
        property: 'initiated_by'
    },
    ModifiedBy: {
        property: 'modified_by'
    },
    UserRemarks: {
        property: 'order_identifier'
    }
}

const conditions = {
    OrderValidity: (mappedValue, sourceObj) => {
        if (sourceObj.Exchange == 2) { // exchange 2: BSE_EQ
            switch (mappedValue) {
                case '1':
                    mappedValue = 'EOSESS';
                    break;
                case '2':
                    mappedValue = 'EOTODY';
                    break;
                case '3':
                    mappedValue = 'EOSTLM';
                    break;
                case '4':
                    mappedValue = 'IOC';
                    break;
            }
            return mappedValue;
        } else {
            switch (mappedValue) {
                case '1':
                    mappedValue = 'DAY';
                    break;
                case '2':
                    mappedValue = 'GTD';
                    break;
                case '3':
                    mappedValue = 'GTC';
                    break;
                case '4':
                    mappedValue = 'IOC';
                    break;
                case '5':
                    mappedValue = 'EOS';
                    break;
                case '6':
                    mappedValue = 'FOK';
                    break;
            }
            return mappedValue;
        }
    },
    OrderType: (mappedValue, sourceObj) => {
        if (!mappedValue) {
            if (sourceObj.OrderType == 11) {
                return sourceObj.Exchange == 5 || sourceObj.Exchange == 6 ? 'RL' : 'CA';
            } else if (sourceObj.OrderType == 12) {
                if (sourceObj.Exchange == 11 || sourceObj.Exchange == 12 || sourceObj.Exchange == 15 || sourceObj.Exchange == 16) {
                    return 'RL';
                }
            }
        } else {
            return mappedValue;
        }
    },
    OrderEntryTime: (mappedValue) => mappedValue ? moment(mappedValue, 'DD-MMM-YYYY HH.mm.ss').format('YYYY-MM-DD HH.mm.ss') : '',
    LastModifiedTime: (mappedValue) => mappedValue ? moment(mappedValue, 'DD-MMM-YYYY HH.mm.ss').format('YYYY-MM-DD HH.mm.ss') : '',
};

function mapResp(sourceObj) {
    const resultObj = {};
    for (const [key, { property, values }] of Object.entries(mappingMsgSocResp)) {
        if (sourceObj.hasOwnProperty(key)) {
            let mappedValue = values ? values[sourceObj[key]] : sourceObj[key];
            if (conditions.hasOwnProperty(key)) {
                mappedValue = conditions[key](mappedValue, sourceObj);
            }
            resultObj[property] = mappedValue;
        }
    }
    return resultObj;
};

function mappedMsgSocResp(sourceObj) {
    let mappedResp = mapResp(sourceObj);
    mappedResp = {
        'message_type': sourceObj['MessageType'],
        'is_amo_order': (sourceObj.Product == 'AD' || sourceObj.Product == 'AM') ? true : false,
        ...mappedResp
    };
    return mappedResp;
}

module.exports = {
    mappedMsgSocResp
}